var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { Swiper, SwiperItem, View } from "@tarojs/components";
import React, { useCallback, useEffect, useMemo, useRef, useState } from "react";
import Taro from "@tarojs/taro";
import SwiperScheduler from "../../SwiperScheduler";
import { getStepValue, sleep } from "../../common/utils";
import { minCount } from "./constant";
import useDeepCompareEffect from "../../hooks/useDeepCompareEffect";
import useMemoizedFn from "../../hooks/useMemoizedFn";
const ENV_TYPE = Taro.getEnv();
function LazySwiper(props) {
    const { className = '', style, vertical, defaultIndex = 0, dataSource, maxCount = 3, loop = false, lazySwiper, duration = 500, renderContent: _renderContent, keyExtractor: _keyExtractor, onChange: _onChange, onBeforeChange: _onBeforeChange, onAnimationFinish: _onAnimationFinish } = props;
    const [isAnimating, setAnimating] = useState(false);
    const [swiperIndex, setSwiperIndex] = useState(0);
    const [source, setSource] = useState([]);
    const [swiperKey, setSwiperKey] = useState('normal');
    const renderContent = useMemoizedFn(_renderContent);
    const keyExtractor = useMemoizedFn(_keyExtractor);
    const onChange = useMemoizedFn(_onChange);
    const onBeforeChange = useMemoizedFn(_onBeforeChange);
    const onAnimationFinish = useMemoizedFn(_onAnimationFinish);
    const updateSwiperIndex = useCallback(async (index) => {
        setSwiperIndex(index);
        setAnimating(true);
        if (ENV_TYPE === 'WEAPP') {
            await sleep(duration);
        }
        swiperSchedulerRef.current.recompute();
        await sleep(Math.floor(duration / 3));
        setAnimating(false);
    }, [duration]);
    const swiperSchedulerRef = useRef(useMemo(() => {
        return new SwiperScheduler({
            defaultMarkIndex: defaultIndex,
            minCount: Math.max(minCount, maxCount),
            loop,
            onSwiperRestart({ swiperIndex: sIndex, key }) {
                setSwiperIndex(sIndex);
                setSwiperKey(key);
            },
            onSwiperIndexChange({ swiperIndex: sIndex }) {
                updateSwiperIndex(sIndex);
            },
            onSwiperSourceChange(value) {
                setSource(value);
            },
            onMarkIndexChange({ markIndex }) {
                onChange({ current: markIndex });
            },
        });
    }, [defaultIndex, maxCount, loop, updateSwiperIndex, onChange]));
    useDeepCompareEffect(() => {
        setSource(swiperSchedulerRef.current.updateDataSource(dataSource));
    }, [dataSource]);
    const updateSwiperIndexByStep = useCallback(async (step) => {
        const swiperScheduler = swiperSchedulerRef.current;
        swiperScheduler.offsetSection(step);
    }, []);
    const handleChange = useCallback((event) => {
        const eventIndex = event.detail.current;
        if (swiperIndex === eventIndex)
            return;
        const step = getStepValue(swiperIndex, eventIndex, source.length - 1);
        updateSwiperIndexByStep(step);
    }, [source.length, swiperIndex, updateSwiperIndexByStep]);
    const handleAnimationFinish = useCallback(() => {
        const swiperScheduler = swiperSchedulerRef.current;
        onAnimationFinish === null || onAnimationFinish === void 0 ? void 0 : onAnimationFinish({ current: swiperScheduler.getDataIndex() });
    }, [onAnimationFinish]);
    const getActiveStatusBySwiperIndex = useCallback((index) => {
        return swiperSchedulerRef.current.getActiveStatusBySwiperIndex(index);
    }, []);
    const canNext = useCallback(async (targetIndex, callback) => {
        const swiperScheduler = swiperSchedulerRef.current;
        const result = await onBeforeChange({ fromIndex: swiperScheduler.getDataIndex(), toIndex: targetIndex });
        if (result === false)
            return;
        callback();
    }, [onBeforeChange]);
    const canNextWithStep = useCallback(async (step, callback) => {
        const swiperScheduler = swiperSchedulerRef.current;
        const targetIndex = swiperScheduler.getDataIndex() + step;
        return canNext(targetIndex, () => callback(targetIndex));
    }, [canNext]);
    const nextSection = useCallback(() => {
        canNextWithStep(1, () => {
            updateSwiperIndexByStep(1);
        });
    }, [canNextWithStep, updateSwiperIndexByStep]);
    const prevSection = useCallback(() => {
        canNextWithStep(-1, () => {
            updateSwiperIndexByStep(-1);
        });
    }, [canNextWithStep, updateSwiperIndexByStep]);
    const toSection = useCallback(async (index) => {
        canNext(index, () => {
            swiperSchedulerRef.current.toSection(index);
        });
    }, [canNext]);
    useEffect(() => {
        if (!lazySwiper)
            return;
        Object.assign(lazySwiper, {
            nextSection,
            prevSection,
            toSection
        });
    }, [lazySwiper, nextSection, prevSection, toSection]);
    return (React.createElement(View, { className: `lazy-swiper ${className}`, style: style },
        React.createElement(Swiper, { className: 'swiper', key: swiperKey, current: swiperIndex, onChange: handleChange, indicatorColor: '#999', indicatorActiveColor: '#333', indicatorDots: true, vertical: vertical, circular: swiperSchedulerRef.current.circular, duration: duration, onAnimationEnd: handleAnimationFinish }, source.map((item, index) => {
            if (!item)
                return;
            const { data } = item, otherProps = __rest(item, ["data"]);
            const isActive = getActiveStatusBySwiperIndex(index);
            const key = keyExtractor(data) || index.toString();
            return (React.createElement(SwiperItem, Object.assign({ key: key }, otherProps), renderContent(data, { key, isActive })));
        })),
        isAnimating ? (React.createElement(View, { className: 'mask' })) : null));
}
export default LazySwiper;
//# sourceMappingURL=LazySwiper.js.map